﻿using AutoMapper;
using ComponentsLibrary.entities;
using ComponentsLibrary.entities.enums;
using Lab3.Database.DTO;
using Lab3.Database.Repository.Interfaces;
using Lab3.Models;

namespace Lab3.Forms
{
    public partial class MainForm : Form
    {
        private readonly IStudentRepository _studentRepository;
        private readonly IMapper _mapper;
        private readonly Func<Guid?, CreateForm> _getCreateOrUpdateForm;
        private readonly CatalogForm _catalogForm;

        private bool _cont = false;

        public MainForm(
            IStudentRepository repository,
            IMapper mapper,
            Func<Guid?, CreateForm> getCreateOrUpdateForm,
            CatalogForm catalogForm)
        {
            _studentRepository = repository;
            _mapper = mapper;

            _getCreateOrUpdateForm = getCreateOrUpdateForm;
            _catalogForm = catalogForm;

            InitializeComponent();
        }

        private void MainForm_LoadAsync(object sender, EventArgs e)
        {
            LoadAsync();
        }

        private async void LoadAsync()
        {
            try
            {
                var students = _mapper.Map<List<StudentViewModel>>(await _studentRepository.GetAsync());
                StudentsListBox.FillValues(
                    students.Select(s => string.Join(" ",
                        [
                            s.Id,
                        s.Name,
                        s.EducationForm,
                        s.StartEducation.ToLongDateString(),
                        s.SessionMarks,
                        ]
                    ))
                );
            }
            catch (Exception ex)
            {
                MessageBox.Show(
                    ex.Message,
                    "Ошибка",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
        }

        private async void RemoveStudentAsync()
        {
            if (MessageBox.Show("Удалить запись", "Вопрос", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                try
                {
                    await _studentRepository
                        .DeleteAsync(Guid.Parse(StudentsListBox.Selected.Split()[0]));
                    LoadAsync();       
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private async void CreateSimpleDocAsync()
        {
            try
            {
                using var saveFileDialog = new SaveFileDialog
                {
                    Filter = "pdf|*.pdf"
                };
                if (saveFileDialog.ShowDialog() != DialogResult.OK)
                {
                    return;
                }

                var values = (await _studentRepository.GetAsync())
                        .Select(s => s.StudentSessions
                            .OrderBy(x => x.Number)
                            .Select(x => x.Score.ToString())
                            .ToArray())
                        .ToArray();
                var tables = new string[values.Length, 6];

                for (int i = 0; i < values.Length; ++i)
                {
                    for (int j = 0; j < 6; ++j)
                    {
                        tables[i, j] = values[i][j]; 
                    }
                }

                SimpleTableCreator.SaveToPdf(new()
                {
                    FilePath = saveFileDialog.FileName,
                    Title = "Сессии",
                    Tables = [tables]
                });

                _ = MessageBox.Show("Простой документ сформирован",
                    "Успешный результат",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                _ = MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private async void CreateTableAsync()
        {
            try
            {
                using var saveFileDialog = new SaveFileDialog
                {
                    Filter = "xlsx|*.xlsx"
                };
                if (saveFileDialog.ShowDialog() != DialogResult.OK)
                {
                    return;
                }
                ExcelTableCreator.SaveToExcel<StudentDTO>(new()
                {
                    FilePath = saveFileDialog.FileName,
                    Title = "Студенты",
                    Headers = 
                    [
                        ("Id", nameof(StudentDTO.Id)),
                        ("ФИО", nameof(StudentDTO.Name)),
                        ("Форма обучения", nameof(StudentDTO.EducationForm)),
                        ("Дата поступления", nameof(StudentDTO.StartEducation)),
                    ],
                    HeaderGroups = [new() {
                        GroupHeader = "Образование",
                        FirstHeader = 2,
                        LastHeader = 3,
                    }],
                    Values = await _studentRepository.GetAsync()
                });

                _ = MessageBox.Show("Документ с таблицей сформирован",
                    "Успешный результат",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                _ = MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private async void CreateWordDocAsync()
        {
            try
            {
                using var saveFileDialog = new SaveFileDialog
                {
                    Filter = "docx|*.docx"
                };
                if (saveFileDialog.ShowDialog() != DialogResult.OK)
                {
                    return;
                }

                var data = (await _studentRepository.GetAsync())
                    .GroupBy(s => s.EducationForm)
                    .Select(s => new DataLine(
                        s.Key,
                        s.GroupBy(x => x.StartEducation.Year)
                        .Select(x => (year: x.Key.ToString(), count: (double)x.Count()))
                        .ToArray()))
                    .ToList();

                WordDiogramCreator.AddDiagram(
                    new(
                        saveFileDialog.FileName,
                        "Поступления",
                        "Диаграмма",

                        EnumAreaLegend.Bottom,
                        data
                    )
                );

                _ = MessageBox.Show("Документ с диаграммой сформирован",
                     "Успешный результат",
                     MessageBoxButtons.OK,
                     MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                _ = MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
        {
            if (keyData == (Keys.ControlKey | Keys.Control)) _cont = true;

            if (_cont && keyData == Keys.A)
            {
                _getCreateOrUpdateForm(null).ShowDialog(this);
                LoadAsync();
            }
            else if (_cont && keyData == Keys.U)
            {
                var elem = StudentsListBox.Selected;
                _getCreateOrUpdateForm(
                        Guid.Parse(elem.Split()[0])
                    ).ShowDialog(this);
                LoadAsync();
            }
            else if (_cont && keyData == Keys.D) 
            {
                RemoveStudentAsync();
            }
            else if (_cont && keyData == Keys.S)
            {
                CreateSimpleDocAsync();
            }
            else if (_cont && keyData == Keys.T)
            {
                CreateTableAsync();
            }
            else if (_cont && keyData == Keys.C)
            {
                CreateWordDocAsync();
            }
            else if (_cont && keyData == Keys.G)
            {
                _catalogForm.ShowDialog(this);
            }

            if (keyData == Keys.A ||
                keyData == Keys.U ||
                keyData == Keys.D ||
                keyData == Keys.S ||
                keyData == Keys.T ||
                keyData == Keys.C ||
                keyData == Keys.G) _cont = false;
            
            return true;
        }
    }
}
