﻿using Lab4.Interfaces;
using Lab4.Models;

namespace Lab4.Forms
{
    public partial class FormMain : Form
    {
        private readonly Dictionary<string, IPluginsConvention> _plugins;
        private string _selectedPlugin;

        private readonly Func<Type, IPluginsConvention> _getPluginObjectFunc;

        public FormMain(Func<Type, IPluginsConvention> getPluginObjectFunc)
        {
            InitializeComponent();
            
            _getPluginObjectFunc = getPluginObjectFunc;

            _plugins = LoadPlugins();
            _selectedPlugin = string.Empty;
        }

        private Dictionary<string, IPluginsConvention> LoadPlugins()
        {
            Dictionary<string, IPluginsConvention> result = [];

            var plurinType = typeof(IPluginsConvention);

            foreach (var type in AppDomain.CurrentDomain
                .GetAssemblies()
                .SelectMany(x => x.GetTypes())
                .Where(x => plurinType.IsAssignableFrom(x) && x != plurinType))
            {
                var plugin = _getPluginObjectFunc(type);

                string key = plugin.PluginName;
                result[key] = plugin;

                var item = new ToolStripMenuItem(key);
                item.Click += (s, e) =>
                {
                    _selectedPlugin = key;

                    panelControl.Controls.Clear();

                    _plugins[_selectedPlugin].ReloadData();
                    var control = _plugins[key].GetControl;
                    control.Parent = panelControl;
                    control.Dock = DockStyle.Fill;

                    panelControl.Controls.Add(control);
                };
                ControlsStripMenuItem.DropDownItems!.Add(item);
            }

            return result;
        }

        private void FormMain_KeyDown(object sender, KeyEventArgs e)
        {
            if (string.IsNullOrEmpty(_selectedPlugin) || !_plugins.ContainsKey(_selectedPlugin))
            {
                return;
            }

            if (!e.Control)
            {
                return;
            }

            switch (e.KeyCode)
            {
                case Keys.I:
                    ShowThesaurus();
                    break;
                case Keys.A:
                    AddNewElement();
                    break;
                case Keys.U:
                    UpdateElement();
                    break;
                case Keys.D:
                    DeleteElement();
                    break;
                case Keys.S:
                    CreateSimpleDoc();
                    break;
                case Keys.T:
                    CreateTableDoc();
                    break;
                case Keys.C:
                    CreateChartDoc();
                    break;
            }
        }

        private void ShowThesaurus()
        {
            _plugins[_selectedPlugin].GetThesaurus()?.Show();
        }

        private void AddNewElement()
        {
            var form = _plugins[_selectedPlugin].GetForm();

            if (form != null && form.ShowDialog() == DialogResult.OK)
            {
                _plugins[_selectedPlugin].ReloadData();
            }
        }

        private void UpdateElement()
        {
            var element = _plugins[_selectedPlugin].GetElement;

            if (element == null)
            {
                _ = MessageBox.Show("Нет выбранного элемента", "Ошибка",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            var form = _plugins[_selectedPlugin].GetForm(element);

            if (form != null && form.ShowDialog() == DialogResult.OK)
            {
                _plugins[_selectedPlugin].ReloadData();
            }
        }
       
        private void DeleteElement()
        {
            if (MessageBox.Show("Удалить выбранный элемент", "Удаление",
            MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
            {
                return;
            }

            var element = _plugins[_selectedPlugin].GetElement;

            if (element == null)
            {
                _ = MessageBox.Show("Нет выбранного элемента", "Ошибка",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (_plugins[_selectedPlugin].DeleteElement(element))
            {
                _plugins[_selectedPlugin].ReloadData();
            }
        }

        private void CreateSimpleDoc()
        {
            using var saveFileDialog = new SaveFileDialog
            {
                Filter = "pdf|*.pdf"
            };
            if (saveFileDialog.ShowDialog() != DialogResult.OK)
            {
                return;
            }

            if (_plugins[_selectedPlugin].CreateSimpleDocument(new PluginsConventionSaveDocument()
            {
                FileName = saveFileDialog.FileName,
            }))
            {
                _ = MessageBox.Show("Документ сохранен",
                    "Создание документа",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Information);
            }
            else
            {
                _ = MessageBox.Show("Ошибка при создании документа", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        
        private void CreateTableDoc()
        {
            using var saveFileDialog = new SaveFileDialog
            {
                Filter = "xlsx|*.xlsx"
            };

            if (saveFileDialog.ShowDialog() != DialogResult.OK)
            {
                return;
            }

            if (_plugins[_selectedPlugin].CreateTableDocument(new
            PluginsConventionSaveDocument()
            {
                FileName = saveFileDialog.FileName,
            }))
            {
                _ = MessageBox.Show("Документ сохранен",
                    "Создание документа",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Information);
            }
            else
            {
                _ = MessageBox.Show("Ошибка при создании документа", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        
        private void CreateChartDoc()
        {
            using var saveFileDialog = new SaveFileDialog
            {
                Filter = "docx|*.docx"
            };

            if (saveFileDialog.ShowDialog() != DialogResult.OK)
            {
                return;
            }

            if (_plugins[_selectedPlugin].CreateChartDocument(new PluginsConventionSaveDocument()
            {
                FileName= saveFileDialog.FileName,
            }))
            {
                _ = MessageBox.Show("Документ сохранен",
                    "Создание документа",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Information);
            }
            else
            {
                _ = MessageBox.Show("Ошибка при создании документа", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        
        private void ThesaurusToolStripMenuItem_Click(object sender, EventArgs e) => ShowThesaurus();
        
        private void AddElementToolStripMenuItem_Click(object sender, EventArgs e) => AddNewElement();
        
        private void UpdElementToolStripMenuItem_Click(object sender, EventArgs e) => UpdateElement();
        
        private void DelElementToolStripMenuItem_Click(object sender, EventArgs e) => DeleteElement();
        
        private void SimpleDocToolStripMenuItem_Click(object sender, EventArgs e) => CreateSimpleDoc();

        private void TableDocToolStripMenuItem_Click(object sender, EventArgs e) => CreateTableDoc();

        private void ChartDocToolStripMenuItem_Click(object sender, EventArgs e) => CreateChartDoc();

    }
}
